function [lambda,density,entropy,NewtonStep] = generalizedME(mu,phi,W,damp,tol,maxIter,lambda0)
% GENERALIZEDME 
% For a given squence of generalized phi-moments mu computes parameters lambda
% of the exponential density function rho = exp(phi*lambda) using 
% moment matching and the damped Newton-Raphson method:
%
% [1] C. Bierig and A. Chernov, Approximation of probability density
% functions by the Multilevel Monte Carlo Maximum Entropy method,
% J. Comput. Physics 314 (2016), 661–681, DOI: 10.1016/j.jcp.2016.03.027
% see also 
% [2] A. Mohammad-Djafari, A MATLAB program to calculate the maximum 
% entropy distributions, in "Maximum Entropy and Bayesian Methods, 
% Max-Ent Proceedings, pp. 221-233, Kluwer Academic Publ., 1991.
%
% input:
%         mu = prescribed moments (column vector)
%        phi = columns of this matrix phi(:,k) contain values of 
%                the polynomial phi of degree k at quadrature points
%          W = quadrature weights (column vector)
%       damp = damping parameter for the damped Newton-Raphson method
%        tol = prescribed tolerance
%    maxIter = upper limit of the number of Newton iterations 
%    lambda0 = initial value for the parameters lambda (column vector)
% output:
%     lambda = computed approximation for the parameters lambda (column vector)
%    density = values of the approximate density at quadrature points (column vector)
%    entropy = convergence history for the Shannon entropy
% NewtonStep = convergence history for the lengths of the Newton increments
%
% Authors: A. Chernov, C. Bierig, alexey.chernov@uni-oldenburg.de
% Date: 28-04-2016
    
    if(nargin <= 3),  damp= 1.0;      end % no damping if damp is not prescribed
    if(nargin <= 4),  tol = 10^-6;    end % initialize tol if not prescribed
    if(nargin <= 5),  maxIter = 10^3; end % initialize maxIter if not prescribed
    if(nargin <= 6)                       % initialize lambda
        lambda=zeros(size(mu));           % This produces a uniform
        lambda(1)=-log(sum(W));           %   distribution on the interval [a,b]
    else                                  %   (recall b-a = sum(W)).
        lambda=lambda0(:);                % set prescribed lambda0
    end

    dlambda = ones(size(mu));             % initialize dlambda with O(1)
    NewtonStep = 1;                       % initialize the length of the Newton step with O(1)
    iter = 1;                             % initialise the iteration counter
  
    while (NewtonStep(max(iter-1,1)) >= tol && iter <= maxIter) % Newton loop       
        densityW = exp(phi*lambda).*W;
        
        F = phi'*densityW - mu;           % current value of the functional
        DF = phi'*diag(densityW)*phi;     % current value of the Jacobi-Matrix
        DF = (DF+DF')/2;                  % enforce symmetry (resulting from roundoff, etc.)
        
        dlambda = -DF\F;                  % compute Newton increment
        lambda = lambda + dlambda*damp;   % Newton update
        
        NewtonStep(iter) = norm(dlambda); % compute the length of the Newton step
        entropy(iter) = -dot(lambda,phi'*densityW);
                                          % display output
        disp(sprintf('iter = %4d | entropy = %23.16e | NewtonStep = %22.16e',...
                                         iter,entropy(iter),NewtonStep(iter)));           
        iter = iter + 1;                  % update iteration counter
    end
    
    density = exp(phi*lambda);            % evaluate the density
end