% This script is a demonstration of the Maximum Entropy method realized by
% matching of generalized moments (Legendre [1], monomial or Fourier moments [2]) 
% on a bounded interval [a,b]:
%
% [1] C. Bierig and A. Chernov, Approximation of probability density
% functions by the Multilevel Monte Carlo Maximum Entropy method,
% J. Comput. Physics 314 (2016), 661–681, DOI: 10.1016/j.jcp.2016.03.027
% see also 
% [2] A. Mohammad-Djafari, A MATLAB program to calculate the maximum 
% entropy distributions, in "Maximum Entropy and Bayesian Methods, 
% Max-Ent Proceedings, pp. 221-233, Kluwer Academic Publ., 1991.
% 
% test_id = 0 approximates the log-normal distrubution on a bounded interval [a,b]. 
% test_id = 1 approximates a smooth, compactly supported in [-1,1] density 
%                  (rho^1 from Table 1, Fig. 1 in [1]) on a bounded interval [a,b]. 
% The moments are approximated by the Monte Carlo simulation.
%
% Authors: A. Chernov, M. Giles, alexey.chernov@uni-oldenburg.de
% Date: 28-04-2016

close all; 
clear variables;

                                            % test example:
test_id = 0;                                % test_id = 0: log-normal density
                                            % test_id = 1: smooth,
                                            % compactly supported in [-1,1]
                                            %    density (rho^1 from Table 1, Fig. 1 in [1])

                                            % Choose the basis:
Poly = @LegendrePArray; figure(1);          %   Legendre polynomials 
% Poly = @MonomialsArray; figure(2);        %   monomials
% Poly = @FourierArray; figure(3);          %   Fourier polynomials (trigonometric functions)

if (test_id == 0)                           % set the simulation parameters dependent on test_id
    a = 0;                                  % interval [a,b] of approximation
    b = 12;                                 %   Caution: [a,b] cannot be too large or too small!!!
    damp = 0.5;                             % Damping parameter for the Newton-Raphson method
    sigma = 0.5;                            % std. deviation of the assoc. normal distribution
    R = 5;                                  % number of stat. moments is R+1
elseif (test_id == 1)
    a = -1;                                 % if [a,b] = [-1,1] Legendre polynomials and monomials  
    b = 1;                                  %    are much better than the Fourier basis
    damp = .5;                     
    R = 10;                                 % number of stat. moments is R+1        
end

N  = 1e7;                                   % parameters of the Monte Carlo
N1 = 100000;                                %   simulation


sums = zeros(R+1,1);                        % initialize the MC estimator 
                                            %   for the moments
for i = 1:N1:N                              % Monte Carlo simulation
  N2 = min(N1,N-i+1);
  if (test_id == 0)
      S  = exp(sigma*randn(N2,1));
  elseif (test_id == 1)
      S = 3.^rand(N2,1)-2;
  end
  sums = sums  + sum(Poly(S,R,a,b),1)';
end

sums = sums / N;                            % average over N simulations

[x,w] = GLquad(5*R,a,b);                    % Gaussian quadrature of sufficiently high order
phi = Poly(x,R,a,b);                        % evaluate polynomials at quadrature points

                                            % Maximum Entropy method
[lambda,p,entr,Nstep] = generalizedME(sums,phi,w,damp,10^-9,1000);

                                            % plot results
t = linspace(a,b,1000)';                    % plot-points at the x-axis
if (test_id == 0)                           % evaluate the exact density at plot-points 
    y = lognpdf(t,0,sigma);                  
elseif (test_id == 1)
    y = (1/log(3)./(t+2)).*(abs(t)<=1);                 
end
plot(t,y,'r');                              % plot exact density    
hold on;
phi = Poly(t,R,a,b);                        % evaluate polynomials at plot-points
density = exp(phi*lambda);                  % evaluate Maximum Entropy density
plot(t,density,'b');                        % plot Maximum Entropy density

legend('exact density','Maximum Entropy approximation');

